@@ -36,7 +36,7 @@ def ConvTranspose2d(x, K, B=None, strides=[2,2], dilations=[1,1], pads=[0,0,0,0]
 def LSTM(X, W, R, B=0, sequence_lens=0, initial_h=0, initial_c=0, hidden_size=None, direction='forward'):
     dirs = {'forward':[1], 'reverse':[-1], 'bidirectional':[1,-1]}
     (L, N, input_dim), hidden_size = X.shape, R.shape[-1]
-    Y = np.zeros((L, len(dirs[direction]), N, hidden_size), dtype=np.float32)
+    Y = np.zeros((L, len(dirs[direction]), N, hidden_size), dtype=X.dtype)
     for i, d in enumerate(dirs[direction]):
         _, H, C = lstm(X, Y[:,i], W[i], R[i], B[i], initial_h[i], initial_c[i], d)
     return Y, H, C
@@ -46,7 +46,7 @@ def ReLU(x):
     return np.multiply(x, x>0, out=x)
 
 def LeakyReLU(x, alpha=0.2):
-    a, b = np.float32(alpha), np.float32(1-alpha)
+    a, b = np.array(alpha, x.dtype), np.array(1-alpha, x.dtype)
     if ep: return ep.evaluate('x*((x>0)*b+a)')
     y = (x>0) * b; y += a; y *= x; return y
 
@@ -86,8 +86,8 @@ def UpSample(x, k, mode='nearest'):
 
 def Resize(x, roi, k, size=None, mode='nearest', 
     coordinate_transformation_mode='half_pixel', nearest_mode='round_prefer_floor'):
-    if k.size == 0: k = size[-2:] // np.array(x.shape[-2:])
-    return upsample(x, k[-2:].astype(int).tolist(), mode, 
+    if k.size == 0: k = size[-2:] / np.array(x.shape[-2:])
+    return upsample(x, k[-2:].tolist(), mode, 
         coordinate_transformation_mode, nearest_mode)
 
 def Concatenate(*xs, axis=0):
@@ -113,6 +113,12 @@ def ReduceSum(x, axis=-1, keepdims=False):
 def ReduceMean(x, axis=-1, keepdims=False):
     return x.mean(axis=tuple(axis), keepdims=keepdims)
 
+def ReduceMax(x, axis=-1, keepdims=False):
+    return x.max(axis=tuple(axis), keepdims=keepdims)
+
+def ReduceMin(x, axis=-1, keepdims=False):
+    return x.min(axis=tuple(axis), keepdims=keepdims)
+
 def BatchNorm(x, K, B):
     if ep: return ep.evaluate('x * K + B')
     x = x * K; x += B; return x
@@ -209,6 +215,19 @@ def InstanceNormalization(x, s, bias, epsilon=1e-5):
     x *= s/var; x += bias - s*mean/var
     return x
 
+def Greater(a, b): return np.greater(a, b)
+
+def NonZero(x): return np.array(np.nonzero(x))
+
+def GreaterOrEqual(a, b): return a >= b
+
+def TopK(x, k, axis=-1, largest=1, sorted=1):
+    idk = np.arange(k) * -largest - (largest>0)
+    idx = np.argsort(x, axis=axis)
+    idx = np.take(idx, idk, axis=axis)
+    vs = np.take_along_axis(x, idx, axis=axis)
+    return vs, idx
+
 def Pad(x, pads, constant_value=0, mode='constant'):
     pads = pads.reshape(2,-1).T.tolist()
     para = {'mode': mode}; 
@@ -231,12 +250,15 @@ layer_map = {'dense': Dense, 'conv': Conv2d, 'relu': ReLU,
              'sub': Sub, 'reducemean': ReduceMean, 'exp': Exp, 'log': Log,
              'mul': Mul, 'gap': GlobalAveragePool, 'pow':Pow, 'matmul': MatMul,
              'identity' : Identity, 'tile': Tile, 'lstm':LSTM,
+             'reducemax':ReduceMax, 'reducemin': ReduceMin,
              'reducesum':ReduceSum, 'div':Div, 'unsqueeze':Unsqueeze, 
              'shape': Shape, 'gather':Gather, 'reshape':Reshape,
              'split':Split, 'tanh':Tanh, 'constantofshape':ConstantofShape,
              'slice':Slice, 'expand':Expand, 'cast':Cast, 'range':Range, 
              'equal':Equal, 'where':Where, 'scatternd':Scatternd,
              'instancenormalization':InstanceNormalization, 'clip':Clip,
+             'greater':Greater, 'nonzero':NonZero, 'greaterorequal':GreaterOrEqual,
+             'topk':TopK,
              'transpose':Transpose, 'logsoftmax':LogSoftmax, 'return':Return}
 
 if __name__ == "__main__": pass
